@@ -13,7 +13,7 @@
 (*  General Public License for more details.                              *)
 (**************************************************************************)
 
-(* $Id: chasen.ml 167 2006-04-24 15:18:14Z yori $ *)
+(* $Id: chasen.ml 183 2006-06-11 17:19:57Z yori $ *)
 
 type darts
 type mrph_prim
@@ -191,6 +191,12 @@ let make_mrph mp =
     Gc.finalise delete_mrph mrph;
     mrph
 
+let make_mrph_data mdp = 
+  let mrph = `Mrph_data mdp in
+    Gc.finalise delete_mrph mrph;
+    mrph
+
+
 let create_undefword keyword headword weight no =
   let undef_info = get_undef_info no in
   let mrph_prim = 
@@ -263,9 +269,7 @@ let delete_mrph_data = function `Mrph_data d -> delete_mrph_data d
 let mrph_data_of_mrph = function 
     `Mrph mrph ->
       let d = mrph_data_of_mrph mrph in
-      let md = `Mrph_data d in
-	Gc.finalise delete_mrph_data md;
-	md
+      make_mrph_data d
   | (`Mrph_data _) as x -> x
     
 let mrph_of_mrph_data ((`Mrph_data _) as x) = x
@@ -312,24 +316,26 @@ let deserialize_mrph string =
     "1,%d,%d,%d,%d,%d,%S,%S,%S,%S,%d,%S,%S,%d"
     (fun posid inf_type	inf_form weight con_tbl keyword headword	
 	reading	pron stem_len base info	compound ->
-	  `Mrph_data (new_mrph_data
-			~posid
-			~inf_type
-			~inf_form
-			~weight
-			~con_tbl
-			~keyword
-			~headword
-			~reading
-			~pron
-			~stem_len
-			~base
-			~info
-			~compound
-			~is_undef:0
-			(* FIXME *)
-			~dat_index:0
-			~darts:(Obj.magic 0)))
+	  let d =
+	    new_mrph_data
+	      ~posid
+	      ~inf_type
+	      ~inf_form
+	      ~weight
+	      ~con_tbl
+	      ~keyword
+	      ~headword
+	      ~reading
+	      ~pron
+	      ~stem_len
+	      ~base
+	      ~info
+	      ~compound
+	      ~is_undef:0
+	      (* FIXME *)
+	      ~dat_index:0
+	      ~darts:(Obj.magic 0) in
+	  make_mrph_data d)
 		
 let new_word ~kaki ~yomi  =
   let mrph_list = ref [] in
@@ -340,24 +346,26 @@ let new_word ~kaki ~yomi  =
   !mrph_list
 
 let null_mrph =
-  `Mrph_data (new_mrph_data
-		~posid:0
-		~inf_type:0
-		~inf_form:0
-		~weight:0
-		~con_tbl:0
-		~keyword:""
-		~headword:""
-		~reading:""
-		~pron:""
-		~stem_len:0
-		~base:""
-		~info:""
-		~compound:0
-		~is_undef:0
-		(* FIXME *)
-		~dat_index:0
-		~darts:(Obj.magic 0))
+  let d =
+    new_mrph_data
+      ~posid:0
+      ~inf_type:0
+      ~inf_form:0
+      ~weight:0
+      ~con_tbl:0
+      ~keyword:""
+      ~headword:""
+      ~reading:""
+      ~pron:""
+      ~stem_len:0
+      ~base:""
+      ~info:""
+      ~compound:0
+      ~is_undef:0
+      (* FIXME *)
+      ~dat_index:0
+      ~darts:(Obj.magic 0) in
+  make_mrph_data d
 
 let compare_mrph m1 m2 =
   let sgn = posid m1 - posid m2 in
