@@ -15,3 +15,7 @@
 
 let personal_dict_file = 
   Filename.concat (Sys.getenv "HOME") ".manadic"
+let total_count_ipa = 10000.0
+let weight_scale = 434.0
+let max_weight = 4000
+let prelude = "/usr/local/src/misc/mana/trunk/mana/mana-prelude.scm"
