@@ -14,3 +14,7 @@
 (**************************************************************************)
 
 val personal_dict_file : string
+val total_count_ipa : float
+val weight_scale : float
+val max_weight : int
+val prelude : string
