@@ -17,64 +17,19 @@
 (* ܥդ *)
 (********************)
 
-module Vertex = struct
-
-  type t = {pos : int; state : int}
-
-  (* üpos,state=-1ɽȤˤ*)
-  let terminal = {pos = -1; state = -1}
-
-  let compare v1 v2 =
-    let sgn = v1.pos - v2.pos in
-    if sgn <> 0 then sgn else
-    let sgn = v1.state - v2.state in
-    sgn
-
-  let equal v1 v2 = (v1.pos = v2.pos) && (v1.state = v2.state)
-
-      (* äȤϥåȤ٤ *)
-  let hash v = v.pos lxor v.state
-
-end
-
-module EdgeLabel = struct 
-
-  (* cost = ǼΤΥ + ξ֤ؤϢܥ *)
-  type t = {mrph : Mrph.mrph; pos : int; cost : int}
-
-  let make_terminal v =
-    {mrph = Mrph.null_mrph; cost = 0; pos = v.Vertex.pos}
-
-  let default = {mrph = Mrph.null_mrph; cost = 0; pos = -1}
-
-  let compare v1 v2 =
-    let sgn = v1.pos - v2.pos in
-    if sgn <> 0 then sgn else
-    let sgn = v1.cost - v2.cost in
-    if sgn <> 0 then sgn else
-    (* IDEA:mrphIDդ롩 *)
-    Mrph.compare_mrph v1.mrph v2.mrph
-
-  let equal v1 v2 = (compare v1 v2 = 0)
-	
-end
+open Trans
 
 (* ܥ *)
-module TransGraph = Graph.Imperative.Digraph.ConcreteLabeled(Vertex)(EdgeLabel)
-
-type parse_obj = TransGraph.t
-
-let create_vertex p s =
-    {Vertex.pos = p; Vertex.state = s}
+type parse_obj = {graph : Graph.t;
+		  start_pos : int; start_state : int;
+		  string : string ;
+		  forward_cost_markers : (int * Edge.t option) Mark.t Mark.t}
 
 let parse string ~state ~pos ~len =
-  (* ĺ֤ȤʬषƳǼȤ *)
-  let vertex_array = Array.make (len+1) [] in
   (* դ֤ɲä *)
-  let graph = TransGraph.create () in
-  let init = {Vertex.pos = pos; Vertex.state = state} in
-  TransGraph.add_vertex graph init;
-  vertex_array.(0) <- [init];
+  let graph = Graph.create (pos+len+1) in
+  let init = Vertex.create ~pos ~state in
+  Graph.add_vertex graph init;
   
   (* posˤΰ֤ǻϤޤǤդä *)
   for i = 0 to len - 1 do
@@ -88,45 +43,35 @@ let parse string ~state ~pos ~len =
       for j = 1  to len - i do
 	let u_list = 
 	  Mrph.undefwords (EucString.sub string (pos + i) j) in
-	r := List.rev_append u_list !r;
+	r := (List.map (fun m -> (m, j)) u_list) @ !r;
       done;
       !r in
 
     (* ĺvĹmlenηmrphɲä롣*)
     let add v (mrph, mlen) =
-      try
-	let s', cost = 
-	  Mrph.connect_mrph ~state:v.Vertex.state ~mrph in
-	let label = {EdgeLabel.mrph = mrph; EdgeLabel.cost = cost; EdgeLabel.pos = v.Vertex.pos} in
-	let v' = create_vertex (pos+i+mlen) s' in
-	let e = TransGraph.E.create v label v' in
-	
-	if not (TransGraph.mem_vertex graph v') then
-	  vertex_array.(i+mlen) <- v' :: vertex_array.(i+mlen);
-	TransGraph.add_edge_e graph e
+      let s', cost = 
+	Mrph.connect_mrph ~state:v.Vertex.state ~mrph in
+      if cost < 0 then () else
+      let label = {EdgeLabel.mrph = mrph; EdgeLabel.cost = cost; EdgeLabel.pos = v.Vertex.pos} in
+      let v' = Vertex.create (pos+i+mlen) s' in
+      let e = Edge.create v label v' in
+      
+      Graph.add_edge graph e
+    in
 
-      with Exit -> () in
-
-    let add_undef v undef = add v (undef, len - i) in
 
     (* ĺv˷Ǥ³ *)
     let expand v =
       List.iter (add v) mrph_list;
-      List.iter (add_undef v) undef_list in
+      List.iter (add v) undef_list in
 
     (* ߤΰ֤ηǤŸ *)
-    List.iter expand vertex_array.(i);
+    Graph.iter_pos expand graph (pos+i)
   done;
-  
-  (* üĤ *)
-  TransGraph.add_vertex graph Vertex.terminal;
-  let add_termlink v =
-    let e = TransGraph.E.create v (EdgeLabel.make_terminal v)
-	Vertex.terminal in
-    TransGraph.add_edge_e graph e in
-  List.iter add_termlink vertex_array.(len);
-  (* ֤Τϥդ *)
-  graph
+
+  {graph=graph; string=string;
+   start_pos=pos; start_state=state;
+   forward_cost_markers=Mark.create ()}   
 
 (**************)
 (* Ŭõ *)  
@@ -142,25 +87,32 @@ module Cost = struct
   let compare = (-)
 end
 
-module PathFinder = Graph.Path.Dijkstra(TransGraph)(Cost)
+module PathFinder = Viterbi(Cost)
 
 (* PathFinder֤ѥ򥹥󤷤ơѤΥꥹȤ *)
-let convert_path pos path =
-  let rec f p cost path' = function
-      [_] -> path'
-    | edge :: r ->
-	let {EdgeLabel.mrph = mrph; EdgeLabel.cost = e_cost} = 
-	  TransGraph.E.label edge in
-	let {Vertex.pos = p'; Vertex.state = s} = TransGraph.E.dst edge in
-	let cost = e_cost + cost in
-	f p' cost ((mrph, p, p' - p, s, cost) :: path') r 
-    | [] -> assert false in
-  List.rev (f pos 0 [] path)  
+let rec convert_path = function
+    [] -> []
+  | (edge, cost) :: rest ->
+      let mrph = (Edge.label edge).EdgeLabel.mrph in
+      let p = (Edge.src edge).Vertex.pos in
+      let {Vertex.pos = p'; Vertex.state = s} = Edge.dst edge in
+      (mrph, p, p'-p, s, cost) :: convert_path rest
 
-let best_path_from_parse_obj graph ~state ~pos =
-    let fv = {Vertex.state = state; Vertex.pos = pos} in
-    let path, cost = PathFinder.shortest_path graph fv Vertex.terminal in
-    convert_path pos path
+let get_forward_cost parse_obj fv =
+    match Mark.lookup parse_obj.forward_cost_markers fv with
+      Some m  -> m
+    | None ->
+	let m = PathFinder.forward_cost parse_obj.graph fv in
+	Mark.mark parse_obj.forward_cost_markers fv m;
+	m
+
+let best_path_from_parse_obj parse_obj ~state ~pos  =
+  let last_pos = Graph.length parse_obj.graph - 1 in
+  let fv = {Vertex.state = state; Vertex.pos = pos} in
+  let forward_cost = get_forward_cost parse_obj fv in
+  let path = 
+    PathFinder.best_path (parse_obj.graph, forward_cost) fv last_pos in
+  convert_path path
 	
 (************)
 (* Ѵ *)
@@ -177,18 +129,22 @@ let list_candidates string ~state ~pos ~mrph_len ~len =
   let cmp (_, _, c1) (_, _, c2) = c1 - c2 in
   List.sort cmp list
 
-let list_candidates_from_parse_obj graph ~state ~pos ~mrph_len =
+let list_candidates_from_parse_obj parse_obj ~state ~pos ~mrph_len =
+  let graph = parse_obj.graph in
   let v = {Vertex.state = state; Vertex.pos = pos} in
-  let edge_list = TransGraph.succ_e graph v in
+  let edge_list = Graph.succ_e graph v in
 
-  let filter (_, label, _) = 
+  let filter e = 
+    let label = Edge.label e in
     EucString.length (Mrph.keyword (label.EdgeLabel.mrph)) = mrph_len in
 
-  let compare (_, label1, _) (_, label2, _) = 
-    label1.EdgeLabel.cost - label2.EdgeLabel.cost in
+  let compare e1 e2 = 
+    (Edge.label e1).EdgeLabel.cost - (Edge.label e2).EdgeLabel.cost in
 
-  let convert (_, label, v2) = 
-    (label.EdgeLabel.mrph, v2.Vertex.state, v2.Vertex.pos) in
+  let convert e  = 
+    let label = Edge.label e in
+    let dst = Edge.dst e in
+    (label.EdgeLabel.mrph, dst.Vertex.state, dst.Vertex.pos) in
 
   (*ͤηѴ *)
   List.map convert 
@@ -201,13 +157,20 @@ let list_candidates_from_parse_obj graph ~state ~pos ~mrph_len =
 (* ؽ *)
 (********)
 
-(* Ǥ *)
-let remove_last list =
-  match List.rev list with
-    [] -> []
-  | _ :: list -> List.rev list
+(*  *)
+let rec list_last = function
+    [] -> raise Not_found
+  | [x] -> x
+  | _ :: rest -> list_last rest
 
-let learn graph ~state ~pos path =
+let learn parse_obj ~state ~pos path =
+
+  if path = [] then () else
+
+  let graph = parse_obj.graph in
+  let last_pos = 
+    let _, p, len = list_last path in
+    p+len in
 
   (* pathˤǰʳϥ̵Ȥ롣*)
   let module Cost = struct
@@ -215,9 +178,6 @@ let learn graph ~state ~pos path =
     type t = Int of int | Inf
 
     let weight label = 
-      let is_terminal = 
-	Mrph.compare_mrph label.EdgeLabel.mrph Mrph.null_mrph = 0 in
-      if is_terminal then Int 0 else
       let pos = label.EdgeLabel.pos in
       let s_form = Mrph.surface_form label.EdgeLabel.mrph in
       let length = EucString.length (Mrph.keyword (label.EdgeLabel.mrph)) in
@@ -243,16 +203,17 @@ let learn graph ~state ~pos path =
       | _, Inf -> -1
   end in
   
-  let module PathFinder = Graph.Path.Dijkstra(TransGraph)(Cost) in
+  let module Learner = Viterbi(Cost) in
 
   let fv = {Vertex.state = state; Vertex.pos = pos} in
+  let forward_cost = Learner.forward_cost graph fv in
 
-  let path, _ =
+  let path  =
     try 
-      PathFinder.shortest_path graph fv Vertex.terminal 
-    with Not_found -> [], Cost.Inf in
-  let f e = 
-    let mrph = (TransGraph.E.label e).EdgeLabel.mrph in
+      Learner.best_path (graph, forward_cost) fv last_pos
+    with Not_found -> [] in
+  let f (e, _) = 
+    let mrph = (Edge.label e).EdgeLabel.mrph in
 (*    Printf.eprintf "%s\n" (Mrph.surface_form mrph); flush stderr;*)
     Mrph.incr_count mrph in
-  List.iter f (remove_last path)
+  List.iter f path
