@@ -13,7 +13,7 @@
 (*  General Public License for more details.                              *)
 (**************************************************************************)
 
-(* $Id: parse.mli 150 2005-09-26 08:52:41Z yori $ *)
+(* $Id: parse.mli 171 2006-04-24 15:23:30Z yori $ *)
 
 (** ʸĹʸΰ֤EUCʸοɽ롣*)
 
@@ -25,18 +25,23 @@ type parse_obj
 val parse : string -> state:int -> pos:int -> len:int -> parse_obj
 
 (** [best_path_from parse_obj state pos]ϰ[pos][state]
- Ŭѥ֤*)
-val best_path_from : parse_obj -> state:int -> pos:int ->
-  (Chasen.mrph * int * int * int * int) list
+ Ŭѥ֤ѥ[(<>,<ϰ>,<Ĺ>,<>,<ѥ>)...]
+ ɽ롣 ѥ¸ߤʤNot_found夲롣 *) 
+val best_path_from_parse_obj : parse_obj -> state:int -> pos:int  -> (Mrph.mrph * int * int * int * int) list
 
 (** [list_candidates string state pos mrph_len len][state]ȯ
 [string][pos]Ĺ[mrph_len]ʬʸбǤ
 롣ͤϷ,Ǥб븶ʸĹ,ηǤ
 Ȥ[(mrph, state, cost)]ΥꥹȤǤ롣*) 
-val list_candidates : string -> state:int -> pos:int -> mrph_len:int -> len:int -> (Chasen.mrph * int * int) list
+val list_candidates : string -> state:int -> pos:int -> mrph_len:int -> len:int -> (Mrph.mrph * int * int) list
 
 (** [list_candidates_from parse_obj state pos mrph_len]
 [list_candidates]Ʊͤ[parse_obj]Ѵ롣 *)
-val list_candidates_from : parse_obj -> state:int -> pos:int -> mrph_len:int ->
-  (Chasen.mrph * int * int) list
+val list_candidates_from_parse_obj : parse_obj -> state:int -> pos:int -> mrph_len:int ->
+  (Mrph.mrph * int * int) list
 
+
+(** [learn parse_obj state pos path] [pos][state]
+ path[path]=[<<ɽع¤>,<ϰ>,<Ĺ>>...]ǤäȲꤷƳ
+ Ԥ *) 
+val learn : parse_obj -> state:int -> pos:int ->  (string * int * int) list -> unit 
