@@ -13,7 +13,7 @@
 (*  General Public License for more details.                              *)
 (**************************************************************************)
 
-(* $Id: personalDict.ml 171 2006-04-24 15:23:30Z yori $ *)
+(* $Id: personalDict.ml 184 2006-06-11 17:20:44Z yori $ *)
 
 let db = 
   let flags = [Gdbm.Dbm_create; Gdbm.Dbm_rdwr] in
@@ -93,9 +93,10 @@ let incr_mrph mrph =
     let float_w' = float_w -. c /. total *. (exp (float_w /. c) -. 1.0) in
     int_of_float float_w' in
   let w' = 
-    if w' < 0 then 0 
+    if w' < 0 then 0
     else if w' > Config.max_weight then Config.max_weight 
-    else w' in
+    else if w - w' > w/2 then w/2 else w
+ in
   Chasen.set_weight mrph w'
 
 let add mrph =
