@@ -1,6 +1,6 @@
 (**************************************************************************)
-(*  Mana : A kana(romaji)-kanji conversion engine using ChaSen algorithm.    *)
-(*  Copyright (C) 2003, 2004, 2005  Yamagata Yoriyuki                     *)
+(*  Mana : A kana(romaji)-kanji conversion engine using ChaSen algorithm. *)
+(*  Copyright (C) 2003, 2004, 2005,2006  Yamagata Yoriyuki                *)
 (*                                                                        *)
 (*  This program is free software; you can redistribute it and/or modify  *)
 (*  it under the terms of the GNU General Public License as published by  *)
@@ -13,14 +13,16 @@
 (*  General Public License for more details.                              *)
 (**************************************************************************)
 
-(* $Id: personalDict.mli 158 2005-11-21 18:16:30Z yori $ *)
+(* $Id: personalDict.mli 171 2006-04-24 15:23:30Z yori $ *)
 
-val lookup : string -> int -> int -> Chasen.mrph list
-val lookup_prefix : string -> int -> int -> (Chasen.mrph * int) list
+(* [(<>,<Ǹ˻Ѥ줿Υ>)...]֤ *)
+val lookup : string -> int -> int -> (Chasen.mrph * int) list
+
+(* [(<>,<Ǹ˻Ѥ줿Υ>,<Ĺ>)...]֤ *)
+val lookup_prefix : string -> int -> int -> (Chasen.mrph * int *int) list
+
+val current_count : unit -> int
 
 val add : Chasen.mrph -> unit
 val remove : Chasen.mrph -> unit
-
-val add_new_word : kaki:string -> yomi:string -> unit
-
-(* val incr_count : Chasen.mrph -> unit*)
+val incr_count : Chasen.mrph -> unit
