@@ -9,8 +9,7 @@ let scheme_stream_from_function = Lexing.from_function
 let parse_stream_with_environment = ScmEval.load 
 
 
-let parse_stream ?(prompt = fun x -> x) scheme_stream = 
-  let env = init_environment () in
+let parse_stream env ?(prompt = fun x -> x) scheme_stream = 
     parse_stream_with_environment env ~prompt scheme_stream
 
 let parse_string env string  =
